import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { GroupAssessmentModel, MyGroupAssessmentModel } from 'src/app/shared/model/group-assessment.model';
import { GroupAssessment1ListModel, MyGroupAssessment1ListModel } from 'src/app/shared/model/group-assessment1-list.model';
import { MyPLModel, PLModel } from 'src/app/shared/model/pl.model';
import { GroupAssessmentService } from 'src/app/shared/services/group-assessment.service';
import { IndicatorsCoursesService } from 'src/app/shared/services/indicators-courses.service';
import { CompetencyIndicatorsCourses1MiniModel, MyCompetencyIndicatorsCourses1MiniModel } from 'src/app/shared/model/competency-indicators-courses1-mini.model';

@Component({
  selector: 'app-edit-group-competencies',
  templateUrl: './edit-group-competencies.component.html',
  styleUrls: ['./edit-group-competencies.component.scss']
})
export class EditGroupCompetenciesComponent {
  @Input() pl: PLModel = new MyPLModel()
  @Output() sendEdit: EventEmitter<boolean> = new EventEmitter<boolean>();
  groupCompetencies: GroupAssessmentModel = new MyGroupAssessmentModel()
  dataSelectList: GroupAssessment1ListModel[] = [];
  dataSelect: GroupAssessment1ListModel = new MyGroupAssessment1ListModel()
  dataList: { check: boolean, data: GroupAssessment1ListModel }[] = []
  dataListModal: { check: boolean, data: CompetencyIndicatorsCourses1MiniModel }[] = []

  search = ""
  currentPage = 1
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  isChecked: boolean = false;
  searchModal = ""
  currentPageModal = 1
  dataLoading = false
  dataModalLoading = false

  constructor(private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private groupAssessmentService: GroupAssessmentService,
    private indicatorsCoursesService: IndicatorsCoursesService
  ) { }

  ngOnInit() {
    this.getGroupAssessmentById()
    this.getIndicatorsCoursesList()
  }
  onEdit() {
    this.sendEdit.emit(false);
  }
  checkSelect() {
    this.dataSelectList = this.dataList.filter(item => item.check).map(item => item.data);
  }

  toggleCheckbox() {
    this.isChecked = !this.isChecked;
    this.dataList.forEach((item) => {
      item.check = this.isChecked;
    });
    this.checkSelect();
  }

  getGroupAssessmentById() {
    this.dataLoading = true;
    this.groupAssessmentService.getById(this.pl.plId).subscribe({
      next: response => {
        this.groupCompetencies = new MyGroupAssessmentModel({ ...response, personalLevel: new MyPLModel(this.pl) })
        this.dataList = new MyGroupAssessmentModel(response).groupAssessment1List.map(x => ({ check: false, data: new MyGroupAssessment1ListModel(x) }))
        this.dataLoading = false;
        this.searchChange();
        this.cdr.detectChanges();
      },
      error: error => {
        this.dataLoading = false;
        this.cdr.detectChanges();
      }
    });
  }

  searchChange() {
    this.currentPage = 1;
    const filteredData = this.dataListFilter();
    this.page = Array.from({ length: Math.ceil(filteredData.length / 10) }, (_, i) => i + 1);
  }
  dataListFilter() {
    return this.dataList
      .filter(x => {
        const data = x.data;
        const match =
          data.competencyIndicatorsCourses1Mini.competencyIndicatorsCourses0.competencyTopic.competencyTopicId.toLowerCase().includes(this.search.toLowerCase()) ||
          data.competencyIndicatorsCourses1Mini.competencyIndicatorsCourses0.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
          data.competencyIndicatorsCourses1Mini.competencyIndicatorsCourses0.expectation.toLowerCase().includes(this.search.toLowerCase());
        return match;
      })
      .sort((a, b) => {
        const topicA = a.data.competencyIndicatorsCourses1Mini.competencyIndicatorsCourses0.competencyTopic.competencyTopicId.toLowerCase();
        const topicB = b.data.competencyIndicatorsCourses1Mini.competencyIndicatorsCourses0.competencyTopic.competencyTopicId.toLowerCase();
        
        const expectationA = a.data.competencyIndicatorsCourses1Mini.competencyIndicatorsCourses0.expectation.toLowerCase();
        const expectationB = b.data.competencyIndicatorsCourses1Mini.competencyIndicatorsCourses0.expectation.toLowerCase();

        return topicA.localeCompare(topicB) || expectationA.localeCompare(expectationB);
      });
  }
  


  getIndicatorsCoursesList() {
    this.dataModalLoading = true
    this.indicatorsCoursesService.getList().subscribe({
      next: response => {
        this.dataListModal = response.map(x => ({ check: false, data: new MyCompetencyIndicatorsCourses1MiniModel(x) }));
        this.dataModalLoading = false;
        this.searchModalChange();
        this.cdr.detectChanges();
      },
      error: error => {
        this.dataModalLoading = false;
        this.cdr.detectChanges();
      }
    });
  }

  selectIndicatorsCourses(data: CompetencyIndicatorsCourses1MiniModel) {
    this.dataList.push({
      check: false, data: new MyGroupAssessment1ListModel({
        competencyIndicatorsCourses1Mini: new MyCompetencyIndicatorsCourses1MiniModel(data)
      })
    })
  }

  searchModalChange() {
    this.currentPage = 1;
    const filteredData = this.dataListModalFilter();
    this.page = Array.from({ length: Math.ceil(filteredData.length / 10) }, (_, i) => i + 1);
  }

  dataListModalFilter() {
    return this.dataListModal.filter(x => {
      const data = x.data
      const match = data.competencyIndicatorsCourses0.competencyTopic.competencyTopicId.toLowerCase().includes(this.searchModal.toLowerCase()) ||
        data.competencyIndicatorsCourses0.tdesc.toLowerCase().includes(this.searchModal.toLowerCase()) ||
        data.competencyIndicatorsCourses0.expectation.toLocaleLowerCase().includes(this.searchModal.toLocaleLowerCase())
      return match;
    });
  }

  saveGroupAssessment1() {
    this.groupCompetencies.groupAssessment1List = this.dataList.map(x => x.data);
    this.groupAssessmentService.post(this.groupCompetencies).subscribe({
      next: (response) => {
        if (response.success) {
          this.showAlert(response.message, 'success');
        } else {
          this.showAlert(response.message, 'error');
        }
      },
      error: (error) => {
        this.showAlert(error.message, 'error');
      }
    });
  }

  deleteGroupAssessmentById() {
    const selectedItems = this.dataList.filter(item => item.check);

    if (selectedItems.length > 0) {
      this.dataList = this.dataList.filter(item => !item.check);
    } else if (this.dataList.length > 0) {
      this.dataList.pop();
    }

    this.cdr.detectChanges();
    this.showAlert('ลบข้อมูลสำเร็จ', 'success');
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }
}
